﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace Test06
{
    class Program
    {
        private static Customer[] Customers;
        private static Product[] products;
        private static Supplier[] Suppliers;

        static void Main(string[] args)
        {
            LoadData();

            // Query samples in Chapter 01
            // Move in separate functions if necessary

            //var query =
            //    from c in Customers
            //    from o in c.Orders
            //    select new { c.Name, c.Orders };

            //var query =
            //    from p in products
            //    where p.IdProduct == 3
            //    from o in p.Orders
            //    select o;

            //var query =
            //    from   c in Customers
            //    join   s in Suppliers
            //           on c.City equals s.City 
            //    select new { c.City, c.Name, SupplierName = s.Name };

            //var query =
            //    from   c in Customers
            //    join   s in Suppliers
            //           on c.City equals s.City
            //           into customerSuppliers
            //    select new { c.City, c.Name, customerSuppliers };

            var query =
                from c in Customers
                select new
                {
                    c.City,
                    c.Name,
                    Products = (from o in c.Orders
                                select new
                                {
                                    o.Product.IdProduct,
                                    o.Product.Price
                                }).Distinct(),
                    CustomerSuppliers = from s in Suppliers
                                        where s.City == c.City
                                        select s
                };

            foreach (var row in query)
            {
                ObjectDumper.Write(row, 2);
            }

            Console.ReadLine();
        }
        static void LoadData()
        {
            Customers = new Customer[] {
                                           new Customer 
                                           {
                                               Name = "Paolo", City = "Brescia", Country = Countries.Italy, Orders = new Order[] {
                                               new Order {Quantity = 3, IdProduct = 1, Shipped = false, Month = "January"},
                                               new Order {Quantity = 5, IdProduct = 2, Shipped = true, Month = "May"}}},
            new Customer {Name = "Marco", City = "Torino", Country = Countries.Italy, Orders = new Order[] {
                new Order {Quantity = 10, IdProduct = 1, Shipped = false, Month = "July"},
                new Order {Quantity = 20, IdProduct = 3, Shipped = true, Month = "December"}}},
            new Customer {Name = "James", City = "Dallas", Country = Countries.USA, Orders = new Order[] {
                new Order {Quantity = 20, IdProduct = 3, Shipped = true, Month = "December"}}},
            new Customer {Name = "Frank", City = "Seattle", Country = Countries.USA, Orders = new Order[] {
                new Order {Quantity = 20, IdProduct = 5, Shipped = false, Month = "July"}}}};

            products = new Product[] {
            new Product {IdProduct = 1, Price = 10 },
            new Product {IdProduct = 2, Price = 20 },
            new Product {IdProduct = 3, Price = 30 },
            new Product {IdProduct = 4, Price = 40 },
            new Product {IdProduct = 5, Price = 50 },
            new Product {IdProduct = 6, Price = 60 }};

            Suppliers = new Supplier[] { 
                new Supplier { Name = "FastDelivery", City = "Dallas" },
                new Supplier { Name = "Horizon", City = "Dallas" },
                new Supplier { Name = "WayFaster", City = "Seattle" },
                new Supplier { Name = "Trucker", City = "Torino" },
            };

            // Complete graph references
            foreach (var c in Customers)
            {
                for (int i = 0; i < c.Orders.Length; i++)
                {
                    c.Orders[i].Product = products[c.Orders[i].IdProduct - 1];
                    c.Orders[i].Customer = c;
                    c.Orders[i].Product.Orders.Add(c.Orders[i]);
                }
            }
        }

    }
    public enum Countries
    {
        USA,
        Italy,
    }

    public class Customer
    {
        public string Name;
        public string City;
        public Countries Country;
        public Order[] Orders;

        public override string ToString()
        {
            return String.Format("{0} - {1} - {2}", this.Name, this.City, this.Country);
        }

        public static Customer Empty
        {
            get
            {
                Customer empty = new Customer();
                empty.Name = String.Empty;
                empty.Country = Countries.Italy;
                empty.City = String.Empty;
                empty.Orders = (new List<Order>(Enumerable.Empty<Order>())).ToArray();

                return (empty);
            }
        }
    }
    public class Supplier
    {
        public string Name;
        public string City;
    }

    public struct Order
    {
        public int Quantity;
        public bool Shipped;
        public string Month;
        public int IdProduct;
        public Product Product;
        public Customer Customer;

        public override string ToString()
        {
            return String.Format("{0} - {1} - {2} - {3}", this.Quantity, this.Shipped, this.Month, this.IdProduct);
        }
        /*
                public override bool Equals(object obj)
                {
                    if (!(obj is Order))
                        return false;
                    else
                    {
                        Order o = (Order)obj;
                        return(o.IdProduct == this.IdProduct && o.Month == this.Month && o.Quantity == this.Quantity && o.Shipped == this.Shipped);
                    }
                }

                public override int GetHashCode()
                {
                    return String.Format("{0}|{1}|{2}|{3}", this.IdProduct, this.Month, this.Quantity, this.Shipped).GetHashCode();
                }
         */
    }

    public class Product
    {
        public int IdProduct;
        public decimal Price;
        public string ProductName;
        public List<Order> Orders = new List<Order>();

        public override string ToString()
        {
            return String.Format("{0} - {1}", this.IdProduct, this.Price);
        }
    }
    /*
     * public class Customer {
            public string Name;
            public string City;
            public Order[] Orders;
        }
        public struct Order {
            public int Quantity;
            public Product Product;
            public Customer Customer;
        }
        public class Product {
            public int IdProduct;
            public decimal Price;
            public string ProductName;
            public Order[] Orders;
        }
     */
}
