﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.EntityClient;
using System.Configuration;
using System.Data;
using System.Data.Common;
using System.Data.Objects;
using System.Transactions;
using System.Data.Objects.SqlClient;
using System.Data.SqlClient;

namespace DevLeap.Linq.LinqToEntities.Querying
{
    class Program
    {
        static void Main(string[] args)
        {
            EntityCommandExplicit();
            EntityCommandExplicitWithValue();
            EntityQueryExplicit();
            EntityQueryTypedExplicit();
            LinqToEntitiesSimpleQuery();
            LinqToEntitiesParametricQuery();
            LinqToEntitiesSingle();
            LinqToEntitiesFirst();
            LinqToEntitiesQueryUsingReverse();
            LinqToEntitiesQueryUsingClientSideFunctions();
            LinqToEntitiesQueryUsingCanonicalFunction();
            LinqToEntitiesQueryUsingDatabaseFunction();
            LinqToEntitiesQueryWithUDF();
            LinqToEntitiesExecuteFunction();
            LinqToEntitiesLazyLoading();
            LinqToEntitiesIncludeMethod();
            LinqToEntitiesLoadMethod();
            LinqToEntitiesLoadPropertyMethod();
            LinqToEntitiesObjectStateManager();
            LinqToEntitiesProjection();
            LinqToEntitiesToTraceString();
            LinqToEntitiesExecuteStoreCommand();
            LinqToEntitiesExecuteStoreQuery();
            LinqToEntitiesTranslateToEntity();
            LinqToEntitiesTranslateToObject();
            LinqToEntitiesCompiledQuery();
            LinqToEntitiesCompiledQueryWithAnonymousTypes();
        }

        static void EntityCommandExplicit()
        {
            EntityConnection cn = new EntityConnection(ConfigurationManager.ConnectionStrings["NorthwindEntities"].ConnectionString);
            EntityCommand cmd = new EntityCommand("SELECT c FROM NorthwindEntities.Customers AS c", cn);

            cn.Open();
            using (EntityDataReader dr = cmd.ExecuteReader(CommandBehavior.SequentialAccess))
            {
                while (dr.Read())
                {
                    Console.WriteLine(((DbDataRecord)dr.GetValue(0))["CustomerID"]);
                }
            }
            cn.Close();
        }

        // Listing 9-1
        static void EntityCommandExplicitWithValue()
        {
            EntityConnection cn = new EntityConnection(ConfigurationManager.ConnectionStrings["NorthwindEntities"].ConnectionString);
            EntityCommand cmd = new EntityCommand("SELECT VALUE c FROM NorthwindEntities.Customers AS c", cn);

            cn.Open();
            using (EntityDataReader dr = cmd.ExecuteReader(CommandBehavior.SequentialAccess))
            {
                while (dr.Read())
                {
                    Console.WriteLine(dr["CustomerID"]);
                }
            }
            cn.Close();
        }

        private static void EntityQueryExplicit()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {

                var customers = db.CreateQuery<DbDataRecord>(
                    "SELECT c FROM NorthwindEntities.Customers AS c");

                foreach (var c in customers)
                {
                    Console.WriteLine(((Customer)c[0]).Display());
                }
            }
        }

        // Listing 9-2
        private static void EntityQueryTypedExplicit()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {

                var customers = db.CreateQuery<Customer>(
                    "SELECT VALUE c FROM NorthwindEntities.Customers AS c");

                foreach (Customer c in customers)
                {
                    Console.WriteLine(c.Display());
                }
            }
        }

        // Listing 9-3
        private static void LinqToEntitiesSimpleQuery()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                var query = from c in db.Customers
                            where c.Country == "Italy"
                            select c;

                foreach (var c in query)
                {
                    Console.WriteLine(c.ContactName);
                }
            }
        }

        // Listing 9-5
        private static void LinqToEntitiesParametricQuery()
        {
            String countryFilter = "Italy";

            using (NorthwindEntities db = new NorthwindEntities())
            {
                var query = from c in db.Customers
                            where c.Country == countryFilter
                            select c;

                foreach (var c in query)
                {
                    Console.WriteLine(c.ContactName);
                }
            }
        }

        private static void LinqToEntitiesSingle()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                Customer alfki = db.Customers.Single(c => c.CustomerID == "ALFKI");
                
                Console.WriteLine("{0} - {1}", 
                    alfki.CustomerID, alfki.ContactName);
            }
        }

        private static void LinqToEntitiesFirst()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                Customer alfki = db.Customers.First(c => c.CustomerID == "ALFKI");

                Console.WriteLine("{0} - {1}",
                    alfki.CustomerID, alfki.ContactName);
            }
        }

        // Listing 9-7
        private static void LinqToEntitiesQueryUsingReverse()
        {
            String countryFilter = "Italy";

            using (NorthwindEntities db = new NorthwindEntities())
            {
                var query = (from c in db.Customers
                             where c.Country == countryFilter
                             select c).Reverse();

                foreach (var c in query)
                {
                    Console.WriteLine(c.ContactName);
                }
            }
        }

        // Listing 9-8
        private static void LinqToEntitiesQueryUsingClientSideFunctions()
        {
            String countryFilter = "Italy";

            using (NorthwindEntities db = new NorthwindEntities())
            {
                var query = from c in db.Customers
                            from o in c.Orders
                            where c.Country == countryFilter
                            && o.OrderDate > DateTime.Now.AddDays(-7)
                            select o;

                foreach (var o in query)
                {
                    Console.WriteLine("{0} - {1}", o.OrderID, o.OrderDate);
                }
            }
        }

        // Listing 9-9
        private static void LinqToEntitiesQueryUsingCanonicalFunction()
        {
            String countryFilter = "Italy";

            using (NorthwindEntities db = new NorthwindEntities())
            {
                var query = from c in db.Customers
                            from o in c.Orders
                            where c.Country == countryFilter
                            && EntityFunctions.DiffDays(o.OrderDate, DateTime.Now) <= 7
                            select new { o.OrderID, o.OrderDate };

                foreach (var o in query)
                {
                    Console.WriteLine("{0} - {1}", o.OrderID, o.OrderDate);
                }
            }
        }

        // Listing 9-11
        private static void LinqToEntitiesQueryUsingDatabaseFunction()
        {
            String countryFilter = "Italy";

            using (NorthwindEntities db = new NorthwindEntities())
            {
                var query = from c in db.Customers
                            from o in c.Orders
                            where c.Country == countryFilter
                            && SqlFunctions.DateAdd("day", 7, o.OrderDate) >= SqlFunctions.GetDate()
                            select new { o.OrderID, o.OrderDate };

                foreach (var o in query)
                {
                    Console.WriteLine("{0} - {1}", o.OrderID, o.OrderDate);
                }
            }
        }

        // Listing 9-15
        private static void LinqToEntitiesQueryWithUDF()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                var query = (from c in db.Customers
                             orderby NorthwindEntities.TotalOrdersAmount(c.CustomerID) descending
                             select new { c.CustomerID, c.CompanyName }).Take(5);

                foreach (var c in query)
                {
                    Console.WriteLine("{0} - {1}", c.CustomerID, c.CompanyName);
                }
            }
        }

        // Listing 9-17
        static void LinqToEntitiesExecuteFunction()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                var top10ExpensiveProducts = db.ExecuteFunction<Ten_Most_Expensive_Products_Result>("Ten_Most_Expensive_Products");

                foreach (var p in top10ExpensiveProducts)
                {
                    Console.WriteLine("{0} - {1}", 
                        p.TenMostExpensiveProducts, p.UnitPrice);
                }
            }
        }

        // Listing 9-18
        static void LinqToEntitiesLazyLoading()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                // db.ContextOptions.LazyLoadingEnabled = false;

                var customers = from c in db.Customers
                                select c;

                foreach (var c in customers)
                {
                    Console.WriteLine("{0}", c.CustomerID);
                    foreach (var o in c.Orders)
                    {
                        Console.WriteLine("{0} - {1}", o.OrderID, o.OrderDate);
                    }
                }
            }
        }

        // Listing 9-20
        static void LinqToEntitiesIncludeMethod()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                var customers = from c in db.Customers
                                select c;

                ObjectQuery<Customer> customersQuery = (customers as ObjectQuery<Customer>).Include("Orders"); //.Include("Orders.Order_Details").Include("Orders.Order_Details.Product");

                foreach (var c in customersQuery)
                {
                    Console.WriteLine("{0}", c.CustomerID);
                    foreach (var o in c.Orders)
                    {
                        Console.WriteLine("{0} - {1}", o.OrderID, o.OrderDate);
                    }
                }
            }
        }

        // Listing 9-23
        static void LinqToEntitiesLoadMethod()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                var customers = from c in db.Customers
                                select c;

                foreach (var c in customers)
                {
                    Console.WriteLine("{0}", c.CustomerID);
                    if (c.Country == "Italy")
                    {
                        if (!c.Orders.IsLoaded)
                            c.Orders.Load();

                        foreach (var o in c.Orders)
                        {
                            Console.WriteLine("{0} - {1}", o.OrderID, o.OrderDate);
                        }
                    }
                }
            }
        }

        // Listing 9-24
        static void LinqToEntitiesLoadPropertyMethod()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                db.ContextOptions.LazyLoadingEnabled = false;

                var orders = from o in db.Orders
                             orderby o.CustomerID
                             select o;

                foreach (var o in orders)
                {
                    Console.WriteLine("{0} - {1}", o.OrderID, o.OrderDate);
                    db.LoadProperty<Order>(o, oc => oc.Customer);
                    if (o.Customer.Country == "Italy")
                    {
                        Console.WriteLine("\tCustomer: {0} - {1}", o.Customer.CustomerID, o.Customer.ContactName);
                    }
                }
            }
        }

        // Listing 9-25
        static void LinqToEntitiesObjectStateManager()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                Customer alfki = db.Customers.FirstOrDefault(c => c.CustomerID == "ALFKI");
                alfki.ContactName = "Brian Johnson";

                var customers = from c in db.Customers
                                select c;

                ObjectQuery<Customer> customersQuery = (customers as ObjectQuery<Customer>);

                // customersQuery.MergeOption = MergeOption.AppendOnly;
                customersQuery.MergeOption = MergeOption.NoTracking;

                foreach (var c in customersQuery)
                {
                    Console.WriteLine("{0} - {1}", c.CustomerID, c.ContactName);
                }
            }
        }

        // Listing 9-26
        static void LinqToEntitiesProjection()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                Customer alfki = db.Customers.FirstOrDefault(c => c.CustomerID == "ALFKI");
                alfki.ContactName = "Brian Johnson";

                var customers = from c in db.Customers
                                select new { c.CustomerID, c.ContactName };

                // ObjectQuery customersQuery = (customers as ObjectQuery);
                // customersQuery.MergeOption = MergeOption.AppendOnly;
                // customersQuery.MergeOption = MergeOption.NoTracking;

                foreach (var c in customers)
                {
                    Console.WriteLine("{0} - {1}", c.CustomerID, c.ContactName);
                }
            }
        }

        // Listing 9-27
        static void LinqToEntitiesToTraceString()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                var customers = from c in db.Customers
                                where c.Country == "Italy"
                                select new { c.CustomerID, c.ContactName };

                Console.WriteLine(((ObjectQuery)customers).ToTraceString());

                foreach (var c in customers)
                {
                    Console.WriteLine("{0} - {1}", c.CustomerID, c.ContactName);
                }
            }
        }

        // Listing 9-28
        static void LinqToEntitiesExecuteStoreCommand()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                Int32 rowsAffected = db.ExecuteStoreCommand(
                    "DELETE FROM Customers WHERE Country = @CountryToDelete",
                    new DbParameter[] { new SqlParameter("CountryToDelete", "Australia") });

                Console.WriteLine("Deleted {0} rows", rowsAffected);
            }
        }

        // Listing 9-29
        static void LinqToEntitiesExecuteStoreQuery()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                ObjectResult<Customer> result = db.ExecuteStoreQuery<Customer>(
                    "SELECT * FROM Customers WHERE Country = @Country",
                    new DbParameter[] { new SqlParameter("Country", "USA") });

                foreach (var c in result)
                {
                    Console.WriteLine("{0} - {1}",
                        c.CustomerID, c.ContactName);
                }
            }
        }

        // Listing 9-30
        static void LinqToEntitiesTranslateToEntity()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                using (SqlConnection cn = new SqlConnection("server=localhost;database=Northwind;integrated security=SSPI;"))
                {
                    cn.Open();
                    SqlCommand cmd = new SqlCommand("SELECT * FROM Customers", cn);

                    using (SqlDataReader dr = cmd.ExecuteReader(CommandBehavior.CloseConnection))
                    {
                        ObjectResult<Customer> result = db.Translate<Customer>(dr);

                        foreach (var c in result)
                        {
                            Console.WriteLine("{0} - {1}",
                                c.CustomerID, c.ContactName);
                        }
                    }
                }
            }
        }

        // Listing 9-31
        private class LightCustomer
        {
            public String CustomerID { get; set; }

            public String ContactName { get; set; }
        }

        static void LinqToEntitiesTranslateToObject()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                using (SqlConnection cn = new SqlConnection("server=localhost;database=Northwind;integrated security=SSPI;"))
                {
                    cn.Open();
                    SqlCommand cmd = new SqlCommand("SELECT CustomerID, ContactName FROM Customers", cn);

                    using (SqlDataReader dr = cmd.ExecuteReader(CommandBehavior.CloseConnection))
                    {
                        ObjectResult<LightCustomer> result = db.Translate<LightCustomer>(dr);

                        foreach (var c in result)
                        {
                            Console.WriteLine("{0} - {1}",
                                c.CustomerID, c.ContactName);
                        }
                    }
                }
            }
        }

        // Listing 9-32
        static readonly Func<NorthwindEntities, String, IQueryable<Customer>>
            compiledSearchCustomersByCountry =
                CompiledQuery.Compile<NorthwindEntities, String, IQueryable<Customer>>(
                (db, countryFilter) => from c in db.Customers
                                       where c.Country == countryFilter
                                       select c);

        private static void LinqToEntitiesCompiledQuery()
        {
            using (NorthwindEntities db = new NorthwindEntities())
            {
                IQueryable<Customer> italianCustomers =
                    compiledSearchCustomersByCountry.Invoke(db, "Italy");

                foreach (var c in italianCustomers)
                {
                    Console.WriteLine(c.CompanyName);
                }

                IQueryable<Customer> germanCustomers =
                    compiledSearchCustomersByCountry.Invoke(db, "Germany");

                foreach (var c in germanCustomers)
                {
                    Console.WriteLine(c.CompanyName);
                }
            }
        }

        // Listing 9-33
        private static void LinqToEntitiesCompiledQueryWithAnonymousTypes()
        {
            var compiledSearchCustomersNamesByCountry =
                    CompiledQuery.Compile((NorthwindEntities db, String countryFilter)
                        => from c in db.Customers
                           where c.Country == countryFilter
                           select new { c.CompanyName, c.ContactName });

            using (NorthwindEntities db = new NorthwindEntities())
            {
                var italianCustomersNames =
                    compiledSearchCustomersNamesByCountry.Invoke(db, "Italy");

                foreach (var c in italianCustomersNames)
                {
                    Console.WriteLine(c);
                }
            }
        }
    }

    static class NorthwindExtender
    {
        public static String Display(this Customer c)
        {
            return (String.Format("{0} - {1}",
                c.CustomerID, c.ContactName));
        }

        public static String Display(this Order o)
        {
            return (String.Format("{0} - {1}",
                o.OrderID, o.ShipName));
        }
    }
}
