﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.Linq;
using System.Data.Linq.Mapping;

namespace DevLeap.Linq.LinqToSql {
    public static class DeferredLoading {
        // 
        // Listing 5-24
        // 
        public static void DemoDeferredLoading() {
            Console.Write("DeferredLoadingEnabled=true  ");
            DemoDeferredLoading(true);
            Console.Write("DeferredLoadingEnabled=false ");
            DemoDeferredLoading(false);
            Console.Write("Using LoadOptions            ");
            DemoLoadWith();
        }

        static void DemoDeferredLoading(bool deferredLoadingEnabled) {
            nwDataContext db = new nwDataContext(Connections.ConnectionString);
            db.DeferredLoadingEnabled = deferredLoadingEnabled;
            //db.Log = Console.Out;

            QueryOrder(db);
        }

        static void DemoLoadWith() {
            nwDataContext db = new nwDataContext(Connections.ConnectionString);
            db.DeferredLoadingEnabled = false;
            //db.Log = Console.Out;

            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Order>(o => o.Order_Details);
            db.LoadOptions = loadOptions;

            QueryOrder(db);
        }

        static void QueryOrder(nwDataContext db) {
            var order = db.Orders.Single((o) => o.OrderID == 10251);
            var orderValue = order.Order_Details.Sum(od => od.Quantity * od.UnitPrice);
            Console.WriteLine(orderValue);
        }

        // 
        // Listing 5-25 (part 1/2)
        // 
        public static void DelayLoadProperty() {
            DataContext db = new DataContext(Connections.ConnectionString);
            Table<DelayCustomer> Customers = db.GetTable<DelayCustomer>();
            db.Log = Console.Out;

            var query =
                from   c in Customers
                where  c.Country == "Italy"
                select c;

            foreach (var row in query) {
                Console.WriteLine(
                    "{0} - {1}",
                    row.CompanyName,
                    row.Address);
            }
        }
    }

    // 
    // Listing 5-25 (part 2/2)
    // 
    [Table(Name = "Customers")]
    public class DelayCustomer {
        private Link<string> _Address;
        
        [Column(IsPrimaryKey = true)]
        public string CustomerID;
        [Column]
        public string CompanyName;
        [Column]
        public string Country;
        [Column(Storage = "_Address")]
        public string Address {
            get { return _Address.Value; }
            set { _Address.Value = value; }
        }
    }
}
