﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Data.Linq;
using System.Data.Linq.Mapping;
using System.ComponentModel;

using DevLeap.Linq.LinqToSql.SimpleQuery;

namespace DevLeap.Linq.LinqToSql.Relationships {
    public class Demo {
        public static void Relationships() {
            DataContext db = new DataContext(Connections.ConnectionString);
            Table<Order> Orders = db.GetTable<Order>();
            db.Log = Console.Out;

            var query =
                from o in Orders
                where o.Customer.Country == "USA"
                select o.OrderID;

            foreach (var row in query) {
                Console.WriteLine(row);

            }
        }
        public static void DeferredLoading() {
            DataContext db = new DataContext(Connections.ConnectionString);
            Table<Order> Orders = db.GetTable<Order>();
            db.Log = Console.Out;

            var query =
                from o in Orders
                where o.OrderID == 10528
                select o;

            foreach (var row in query) {
                Console.WriteLine(row.Customer.Country);
            }
        }
        public static void ChildrenAccess() {
            DataContext db = new DataContext(Connections.ConnectionString);
            Table<Customer> Customers = db.GetTable<Customer>();
            db.Log = Console.Out;

            var query =
                from c in Customers
                where c.Country == "Switzerland"
                select c;

            foreach (var row in query) {
                Console.WriteLine(row.CompanyName);
                foreach (var order in row.Orders) {
                    Console.WriteLine(order.OrderID);
                }
            }
        }
        public static void ChildrenQuery_Simple() {
            DataContext db = new DataContext(Connections.ConnectionString);
            Table<Customer> Customers = db.GetTable<Customer>();
            db.Log = Console.Out;

            var query =
                from c in Customers
                where c.Orders.Count > 20
                select c.CompanyName;

            foreach (var row in query) {
                Console.WriteLine(row);
            }
        }
        public static void ChildrenQuery_DeferredLoading() {
            DataContext db = new DataContext(Connections.ConnectionString);
            Table<Customer> Customers = db.GetTable<Customer>();
            db.Log = Console.Out;

            var query =
                from c in Customers
                where c.Orders.Count > 20
                select c;

            foreach (var row in query) {
                Console.WriteLine(row.CompanyName);
                foreach (var order in row.Orders) {
                    Console.WriteLine(order.OrderID);
                }
            }
        }

        // 
        // Listing 5-9
        // 
        public static void ChildrenQuery_ExplicitImmediateLoading() {
            DataContext db = new DataContext(Connections.ConnectionString);
            Table<Customer> Customers = db.GetTable<Customer>();

            db.Log = Console.Out;

            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.LoadWith<Customer>(c => c.Orders);
            db.LoadOptions = loadOptions;
            var query =
                 from c in Customers
                 where c.Orders.Count > 20
                 select c;

            foreach (var row in query) {
                Console.WriteLine(row.CompanyName);
                foreach (var order in row.Orders) {
                    Console.WriteLine(order.OrderID);
                }
            }
        }

        public static void ChildrenQuery_ImplicitImmediateLoading() {
            DataContext db = new DataContext(Connections.ConnectionString);
            Table<Customer> Customers = db.GetTable<Customer>();
            db.Log = Console.Out;

            var query =
                from c in Customers
                where c.Orders.Count > 20
                select new { c.CompanyName, c.Orders };

            foreach (var row in query) {
                Console.WriteLine(row.CompanyName);
                foreach (var order in row.Orders) {
                    Console.WriteLine(order.OrderID);
                }
            }
        }


        // 
        // Listing 5-10
        // 
        public static void ChildrenQuery_UsingAssociateWith() {
            DataContext db = new DataContext(Connections.ConnectionString);
            Table<Customer> Customers = db.GetTable<Customer>();
            db.Log = Console.Out;

            DataLoadOptions loadOptions = new DataLoadOptions();
            loadOptions.AssociateWith<Customer>(
                c => from    o in c.Orders 
                     where   o.OrderDate.Value.Year == 1997
                     orderby o.OrderDate descending
                     select  o);
            // Uncomment the following line to load all the orders that are filtered
            // by previous AssociateWith statement
            // loadOptions.LoadWith<Customer>(c => c.Orders); 
            db.LoadOptions = loadOptions;
            var query =
                 from c in Customers
                 where c.Orders.Count > 5
                 select c;

            foreach (var row in query) {
                Console.WriteLine(row.CompanyName);
                foreach (var order in row.Orders) {
                    Console.WriteLine(order.OrderID);
                }
            }
        }
    }

    //
    // Listing 5-7 and Listing 5-8
    //
    [Table(Name = "Customers")]
    public class Customer {
        [Column(IsPrimaryKey = true)]
        public string CustomerID;
        [Column]
        public string CompanyName;
        [Column]
        public string Country;

        // The following is the code for Listing 5-7
        //[Association(OtherKey="CustomerID")]
        //public EntitySet<Order> Orders;

        // The following is the code for Listing 5-8
        private EntitySet<Order> _Orders;

        [Association(OtherKey = "CustomerID", Storage = "_Orders")]
        public EntitySet<Order> Orders {
            get { return this._Orders; }
            set { this._Orders.Assign(value); }
        }

        public Customer() {
            this._Orders = new EntitySet<Order>();
        }
    }

    //
    // Listing 5-6
    //
    [Table(Name = "Orders")]
    public class Order {
        [Column(IsPrimaryKey = true)]
        public int OrderID;
        [Column]
        private string CustomerID;
        [Column]
        public DateTime? OrderDate;

        [Association(Storage = "_Customer", ThisKey = "CustomerID", IsForeignKey=true)]
        public Customer Customer {
            get { return this._Customer.Entity; }
            set { this._Customer.Entity = value; }
        }

        private EntityRef<Customer> _Customer;

        public Order() {
            _Customer = default(EntityRef<Customer>);
        }
    }
}
